@@ -46,16 +46,16 @@ showAboutDialog window = do
     dialog `set` [ aboutDialogName := __ "Bustle"
                  , aboutDialogVersion := showVersion version
                  , aboutDialogComments := __ "Someone's favourite D-Bus profiler"
-                 , aboutDialogWebsite := "http://willthompson.co.uk/bustle"
+                 , aboutDialogWebsite := "http://www.freedesktop.org/wiki/Software/Bustle/"
                  , aboutDialogAuthors := authors
-                 , aboutDialogCopyright := "© 2008–2014 Will Thompson, Collabora Ltd. and contributors"
+                 , aboutDialogCopyright := "© 2008–2015 Will Thompson, Collabora Ltd. and contributors"
                  , aboutDialogLicense := license
+                 , aboutDialogLogoIconName := Just "bustle"
+                 , windowModal := True
+                 , windowTransientFor := window
                  ]
-    dialog `afterResponse` \resp ->
+    dialog `after` response $ \resp ->
         when (resp == ResponseCancel) (widgetDestroy dialog)
-    windowSetTransientFor dialog window
-    windowSetModal dialog True
-    aboutDialogSetLogoIconName dialog (Just "bustle")
 
     widgetShowAll dialog
 
